/*
 * Project:        Exercise 10.2
 * File:           PoissonModel.cxx
 * Author:         Kevin Kroeninger
 * Version (date): 1.0 (21.06.2013)
 *
 * Copyright (C) 2013, Kevin Kroeninger
 * All rights reserved.
 *
 * Description:
 * A counting model formulated in BAT version 0.9.2. BAT must be
 * installed. The code has to be compiled. 
 *
 * This code is distributed with the solution manual to the book
 *
 * Data Analysis in High Energy Physics: A Practical Guide to Statistical Methods,
 * Wiley-VCH (2013),
 * O. Behnke, K. Kroeninger, G. Schott, Th. Schorrner-Sadenius (editors)
 */

#include "PoissonModel.h"

#include <TMath.h>
#include <BAT/BCMath.h>

// ---------------------------------------------------------
PoissonModel::PoissonModel() : BCModel("poisson")
			     , fNObs(0)
			     , fNuBkg(0)
{
  DefineParameters();
};

// ---------------------------------------------------------
PoissonModel::~PoissonModel()
{
};

// ---------------------------------------------------------
void PoissonModel::DefineParameters()
{
  // add parameters
  AddParameter("#nu_{sgn}", 0., 7.); // index 0
  AddParameter("#nu_{bkg}", 0., 7.); // index 1
}

// ---------------------------------------------------------
int PoissonModel::SetNObs(int nobs, double nu_bkg)
{
  // check that number is positive and greater than 0
  if (nobs < 0)
    return 0;

  // set number of observed events
  fNObs = nobs;

  // set number of expected background events
  fNuBkg = nu_bkg;

  // adjust parameter ranges
  double nu_sgn_min = 0;
  double nu_sgn_max = 10;

  // the adjustment depends on the number of observed events
  if (nobs >= 5 && nobs < 10) {
    nu_sgn_min = 0;
    nu_sgn_max = 20;
  }
  else if (nobs >= 10 && nobs < 20) {
    nu_sgn_min = 0;
    nu_sgn_max = 40;
  }
  else if (nobs >= 20 && nobs < 30) {
    nu_sgn_min = 5;
    nu_sgn_max = 55;
  }
  else if (nobs >= 30) {
    nu_sgn_min = double(nobs) - 5*sqrt(double(nobs));
    nu_sgn_max = double(nobs) + 5*sqrt(double(nobs));
  }

  // re-set the parameter range
  SetParameterRange(0, TMath::Max(nu_sgn_min-nu_bkg, 0.), nu_sgn_max-nu_bkg);
  SetParameterRange(1, nu_bkg-5*0.1*nu_bkg, nu_bkg+5*0.1*nu_bkg);

  // no error
  return 1;
}

// ---------------------------------------------------------
double PoissonModel::LogLikelihood(const std::vector<double> &parameters)
{
  double logprob = 0.;

  // total number of expected events
  double nu = parameters.at(0) + parameters.at(1);

  // log Poisson term
  logprob += BCMath::LogPoisson(double(fNObs), nu);

  // return log likelihood
  return logprob;
}

// ---------------------------------------------------------
