/*
 * Project:        Exercise 10.2
 * File:           runPoissonCounting.cxx
 * Author:         Kevin Kroeninger
 * Version (date): 1.0 (21.06.2013)
 *
 * Copyright (C) 2013, Kevin Kroeninger
 * All rights reserved.
 *
 * Description:
 * A counting model formulated in BAT version 0.9.2. BAT must be
 * installed. The code has to be compiled. 
 *
 * This code is distributed with the solution manual to the book
 *
 * Data Analysis in High Energy Physics: A Practical Guide to Statistical Methods,
 * Wiley-VCH (2013),
 * O. Behnke, K. Kroeninger, G. Schott, Th. Schorrner-Sadenius (editors)
 */

#include "PoissonModel.h"

#include <BAT/BCLog.h>
#include <BAT/BCAux.h>
#include <BAT/BCH1D.h>
#include <BAT/BCH2D.h>
#include <BAT/BCSummaryTool.h>

#include <TH1D.h>
#include <TH2D.h>
#include <TCanvas.h> 
#include <TRandom3.h>

int main()
{

  // set nicer style for drawing than the ROOT default
  BCAux::SetStyle();

  // open log file
  BCLog::OpenLog("log.txt");
  BCLog::SetLogLevel(BCLog::detail);

  // set number of ensembles
  int nensembles = 100;

  // set run time
  //  double time = 1; // in months
  //double time = 1*12; // in months
  double time = 1*12*5; // in months

  // calculate background expectation
  double nu_bkg = 1.5 * time;

  TH1D* hist_limit = new TH1D("hist_limit", ";95% limit;N", 12, 0., 60.);
  hist_limit->SetStats(kFALSE);
  TH1D* hist_nobs = new TH1D("hist_nobs", ";N_{obs};N", 100, nu_bkg-5*sqrt(nu_bkg), nu_bkg+5*sqrt(nu_bkg));

  TRandom3* rand = new TRandom3(1001);

  // study a)

  for (int i = 0 ; i < nensembles; ++i) {

    // draw random number of observed events
    int nobs = rand->Poisson(nu_bkg);

    // create new PoissonModel object
    PoissonModel* m = new PoissonModel();

    // set number of observed events
    m->SetNObs(nobs, nu_bkg);
		
    // choose priors
    m->SetPriorConstant(0);      // uniform prior for signal parameter
    m->SetPriorDelta(1, nu_bkg); // delta prior for background parameter
		
    // set precision
    m->MCMCSetPrecision(BCEngineMCMC::kMedium);
		
    // calculate marginal distributions
    m->MarginalizeAll();

    // calculate limit
    double limit = m->GetMarginalized("#nu_{sgn}")->GetQuantile(0.95);

    // print
    m->PrintAllMarginalized("PoissonModel_plots.pdf");
    m->PrintResults("PoissonModel_results.txt");

    // fill histograms
    hist_limit->Fill(limit);
    hist_nobs->Fill(nobs);

    // free memory
    delete m;
  }

  // studies b) and c)

  // set run time
  time = 1*12*4; // in months

  // calculate background expectation
  nu_bkg = 1.5 * time;	

  // set number of observed events
  int nobs = 100;

  // create new PoissonModel object
  PoissonModel* m_100 = new PoissonModel();

  // set number of observed events
  m_100->SetNObs(nobs, nu_bkg);
	
  // choose priors
  m_100->SetPriorConstant(0);      // uniform prior for signal parameter
  m_100->SetPriorDelta(1, nu_bkg); // delta prior for background parameter
	
  // set precision
  m_100->MCMCSetPrecision(BCEngineMCMC::kMedium);
	
  // calculate marginal distributions
  m_100->MarginalizeAll();
	
  // calculate limit
  double limit = m_100->GetMarginalized("#nu_{sgn}")->GetQuantile(0.95);
	
  std::cout << " limit = " << limit << std::endl;

  // plot histograms and results
  m_100->GetMarginalized("#nu_{sgn}")->Print("ex_10_2_b1.pdf", "BTulB1CS1L", 0.95);
  m_100->PrintAllMarginalized("PoissonModel_plots.pdf");
  m_100->PrintResults("PoissonModel_results.txt");
	
  // free memory
  delete m_100;

  // create new PoissonModel object
  PoissonModel* m_100_prior = new PoissonModel();

  // set number of observed events
  m_100_prior->SetNObs(nobs, nu_bkg);
	
  // choose priors
  m_100_prior->SetPriorConstant(0);      // uniform prior for signal parameter
  m_100_prior->SetPriorGauss(1, nu_bkg, 0.1*nu_bkg); // delta prior for background parameter
	
  // set precision
  m_100_prior->MCMCSetPrecision(BCEngineMCMC::kMedium);
	
  // calculate marginal distributions
  m_100_prior->MarginalizeAll();
	
  // calculate limit
  limit = m_100_prior->GetMarginalized("#nu_{sgn}")->GetQuantile(0.95);
	
  std::cout << " limit = " << limit << std::endl;

  // calculate correlation
  double rho = m_100_prior->GetMarginalized("#nu_{sgn}", "#nu_{bkg}")->GetHistogram()->GetCorrelationFactor();
	
  std::cout << " rho   = " << rho << std::endl;

  // plot histograms and results
  m_100_prior->GetMarginalized("#nu_{sgn}")->Print("ex_10_2_c1.pdf", "BTulB1CS1L", 0.95);
  m_100_prior->GetMarginalized("#nu_{sgn}", "#nu_{bkg}")->Print("ex_10_2_c2.pdf", "BTcB3CS1gmode");
  m_100_prior->PrintAllMarginalized("PoissonModel_prior_plots.pdf");
  m_100_prior->PrintResults("PoissonModel_prior_results.txt");

  // free memory
  delete m_100_prior;

  for (int i = 0 ; i < nensembles; ++i) {

    // draw random number of observed events
    int nobs = rand->Poisson(nu_bkg);

    // create new PoissonModel object
    PoissonModel* m = new PoissonModel();

    // set number of observed events
    m->SetNObs(nobs, nu_bkg);
		
    // choose priors
    m->SetPriorConstant(0);      // uniform prior for signal parameter
    m->SetPriorDelta(1, nu_bkg); // delta prior for background parameter
		
    // set precision
    m->MCMCSetPrecision(BCEngineMCMC::kMedium);
		
    // calculate marginal distributions
    m->MarginalizeAll();

    // calculate limit
    double limit = m->GetMarginalized("#nu_{sgn}")->GetQuantile(0.95);

    // fill histograms
    hist_limit->Fill(limit);
    hist_nobs->Fill(nobs);

    // free memory
    delete m;
  }
	

  TCanvas* c1 = new TCanvas("c1");
  c1->cd();
  hist_limit->Draw();
  c1->Print("ex_10_2_b2.pdf");

  hist_nobs->Draw();
  c1->Print("nobs.pdf");

  std::cout << " Summary: " << std::endl;
  std::cout << " Time                   = " << time << " months" << std::endl;
  std::cout << " Background expectation = " << nu_bkg << " events" << std::endl;
  std::cout << " Avg. number of events  = " << hist_nobs->GetMean() << std::endl;
  std::cout << " Most likely limit      = " << hist_limit->GetBinCenter( hist_limit->GetMaximumBin()) << std::endl;
  std::cout << " Avg. 95% limit         = " << hist_limit->GetMean() << std::endl;
  std::cout << " Std. 95% limit         = " << hist_limit->GetRMS() << std::endl;
  std::cout << " Most likely limit      = " << hist_limit->GetBinCenter(hist_limit->GetMaximumBin()) << std::endl;
  std::cout << " Number of ensembles    = " << nensembles << std::endl;
  std::cout << std::endl;

  // close log file
  BCLog::CloseLog();

  // no error
  return 0;

}

