/*
 * Project:        Exercise 10.3
 * File:           runTemplate.cxx
 * Author:         Kevin Kroeninger
 * Version (date): 1.0 (21.06.2013)
 *
 * Copyright (C) 2013, Kevin Kroeninger
 * All rights reserved.
 *
 * Description:
 * A template fit to the data formulated in BAT version 0.9.2. BAT must
 * be installed. The code has to be compiled.
 *
 * This code is distributed with the solution manual to the book
 *
 * Data Analysis in High Energy Physics: A Practical Guide to Statistical Methods,
 * Wiley-VCH (2013),
 * O. Behnke, K. Kroeninger, G. Schott, Th. Schorrner-Sadenius (editors)
 */

#include "Template.h"

#include <BAT/BCLog.h>
#include <BAT/BCAux.h>
#include <BAT/BCH1D.h>
#include <BAT/BCSummaryTool.h>

#include <TH1D.h>
#include <TCanvas.h>
#include <TRandom3.h>
#include <TMath.h>

int main()
{

  // set nicer style for drawing than the ROOT default
  BCAux::SetStyle();

  // open log file
  BCLog::OpenLog("log.txt");
  BCLog::SetLogLevel(BCLog::detail);

  // intialize random number generator
  TRandom3* rand = new TRandom3(1014);

  // define histograms
  double mean     = 2039;
  double width    =    5;
  double binwidth =    5;
  int nbins       =   11;
  double xmin = mean - (double(nbins-1)/2.+0.5)*binwidth;
  double xmax = mean + (double(nbins-1)/2.+0.5)*binwidth;
  TH1D* hist_data       = new TH1D("hist_data",       ";;", nbins, xmin, xmax);
  TH1D* hist_signal     = new TH1D("hist_signal",     ";;", nbins, xmin, xmax);
  TH1D* hist_background = new TH1D("hist_background", ";;", nbins, xmin, xmax);

  // calculate number of expected events
  double exp_bkg = 1.5 * 12 * 4;

  // fill data histogram
  for (int i = 0; i < int(exp_bkg); ++i) {
    double E = rand->Uniform(xmin, xmax);
    hist_data->Fill(E);
  }

  // fill signal and background histograms
  for (int i = 1; i <= nbins; ++i) {
    double x = hist_signal->GetBinCenter(i);
    double s = 1./sqrt(2.*TMath::Pi())/width * exp( -(x-mean)*(x-mean)/2./width/width );
    double b = 1;
    hist_signal->SetBinContent(i, s);
    hist_background->SetBinContent(i, b);
  }
  hist_signal->Scale(1./hist_signal->Integral());
  hist_background->Scale(1./hist_background->Integral());
   
  // print histograms
  TCanvas* c1 = new TCanvas("c1");
  hist_data->Draw();
  c1->Print("data.eps");
  hist_signal->Draw();
  c1->Print("signal.eps");
  hist_background->Draw();
  c1->Print("background.eps");

  // create new Template object
  Template * m = new Template();

  // set precision
  m->MCMCSetPrecision(BCEngineMCMC::kMedium);

  // set histograms
  m->SetData(hist_data);
  m->SetSignal(hist_signal);
  m->SetBackground(hist_background);

  // set background expectation
  m->SetBkg(exp_bkg);

  for (int i = 0; i < 3; ++i) {
    // set LH option
    int option = i;
    m->SetLHOption(option);
     
    m->MarginalizeAll();

    m->FindMode( m->GetBestFitParameters() );

    m->PrintAllMarginalized(Form("Template_plots_%i.ps", option));

    m->GetMarginalized("S")->Print(Form("Signal_%i.pdf", i), "BTulB1CS1L", 0.95);

    m->PrintResults(Form("Template_results_%i.txt", option));

    // print to screen
    std::cout << " option " << option << " : " << m->GetMarginalized("S")->GetQuantile(0.95) << std::endl;
  }

  // free memobry
  delete m;

  // close log file
  BCLog::CloseLog();

  return 0;

}

